'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
/* eslint-disable
  no-new-func
*/
var toType = function toType(value) {
  return Object.prototype.toString.call(value).slice(8, -1);
};
var replace = function replace(target, replacer, _defaults, _key) {
  var object = void 0;
  var val = replacer(target, _key);
  var type = toType(val);

  if (type === 'Object') {
    object = Object.create(_defaults || {});
  } else if (type === 'Array') {
    object = [].concat(_defaults || []);
  } else {
    object = val;
  }

  if (type === 'Object' || type === 'Array') {
    for (var index in val) {
      if (Object.prototype.hasOwnProperty.call(val, index)) {
        object[index] = replace(val[index], replacer, object[index], index);
      }
    }
    return object;
  }

  return object;
};

var encode = exports.encode = function encode(options) {
  return replace(options, function (value) {
    var type = toType(value);
    if (type === 'RegExp' || type === 'Function') {
      return `<${type}|${value.toString()}>`;
    }
    return value;
  });
};

var decode = exports.decode = function decode(options) {
  return replace(options, function (value, key) {
    var type = toType(value);

    if (type === 'String') {
      var reg = /^<(\w+)\|([\w\W]*)>$/;
      var match = value.match(reg);
      if (match) {
        if (match[1] === 'RegExp') {
          return Function(`return ${match[2]}`)();
        } else if (match[1] === 'Function') {
          // TODO Use ESlint to pre-check options
          return Function(`
          try {
            return (${match[2]}).apply(null, arguments);
          } catch(err) {
            if (err instanceof ReferenceError && ~err.message.indexOf('is not defined')) {
              throw new Error('If the option "${key}" is a function, and it relies on external var, may not work in multi-process mode after serialization: ' + err.message);
            } else {
              throw err;
            }
          }
        `);
        }
      }
    }

    return value;
  });
};